const nsIPermissionManager = Components.interfaces.nsIPermissionManager;
const nsICookie = Components.interfaces.nsICookie;
const ALLOW_SESSION_ONLY = 8; // XXX this needs to be defined publicly, see bug 225857 comment 7

var kObserverService;
var cookiemanager = null;
var gDateService = null;
var cookies = [];
var deletedCookies = [];
var cookieBundle;
var cookiesTree;
var lastCookieSortColumn = "";
var lastCookieSortAscending = false;

var cookiesTreeView = {
  rowCount : 0,
  setTree : function(tree){},
  getImageSrc : function(row,column) {},
  getProgressMode : function(row,column) {},
  getCellValue : function(row,column) {},
  getCellText : function(row,column){
    var rv="";
    if (column=="domainCol") {
      rv = cookies[row].rawHost;
    } else if (column=="nameCol") {
      rv = cookies[row].name;
    } else if (column=="statusCol") {
      rv = GetStatusString(cookies[row].status);
    }
    return rv;
  },
  isSeparator : function(index) {return false;},
  isSorted: function() { return false; },
  isContainer : function(index) {return false;},
  cycleHeader : function(aColId, aElt) {},
  getRowProperties : function(row,column,prop){},
  getColumnProperties : function(column,columnElement,prop){},
  getCellProperties : function(row,prop){}
 };

function Startup() {
  // arguments passed to this routine:
  //   cookieManager 
  //   cookieManagerFromIcon
 
  cookiemanager = Components.classes["@mozilla.org/cookiemanager;1"].getService();
  cookiemanager = cookiemanager.QueryInterface(Components.interfaces.nsICookieManager);

  if (!gDateService) {
    gDateService = Components.classes["@mozilla.org/intl/scriptabledateformat;1"]
                             .getService(Components.interfaces.nsIScriptableDateFormat);
  }
  cookieBundle = document.getElementById("cookieBundle");
  cookiesTree = document.getElementById("cookiesTree");
  var element = document.getElementById("cookiesTab");
  loadCookies();
  // be prepared to reload the display if anything changes
  kObserverService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  kObserverService.addObserver(cookieReloadDisplay, "cookieChanged", false);
}

function Shutdown()
{
  kObserverService.removeObserver(cookieReloadDisplay, "cookieChanged");
}

var cookieReloadDisplay = {
  observe: function(subject, topic, state) {
    if (topic == "cookieChanged") {
      if (state == "cookies") {
        cookies.length = 0;
        if (lastCookieSortColumn == "rawHost") {
          lastCookieSortAscending = !lastCookieSortAscending; // prevents sort from being reversed
        }
        loadCookies();
      }
    }
  }
}

function Cookie(number, name, value, isDomain, host, rawHost, path, isSecure, expires, status, policy)
{
  this.number = number;
  this.name = name;
  this.value = value;
  this.isDomain = isDomain;
  this.host = host;
  this.rawHost = rawHost;
  this.path = path;
  this.isSecure = isSecure;
  this.expires = expires;
  this.status = status;
  this.policy = policy;
}

function loadCookies()
{
  var count = 0;
  var showPolicyField = false;
  var enumerator = cookiemanager.enumerator;

  while (enumerator.hasMoreElements()) {
    var nextCookie = enumerator.getNext();

    if (!nextCookie) 
      break;
    nextCookie = nextCookie.QueryInterface(Components.interfaces.nsICookie);
    var host = nextCookie.host;

    if (nextCookie.policy != nsICookie.POLICY_UNKNOWN) {
      showPolicyField = true;
    }
    cookies[count] = new Cookie(count++, nextCookie.name, nextCookie.value, nextCookie.isDomain, host,
                                (host.charAt(0)==".") ? host.substring(1,host.length) : host,
                                nextCookie.path, nextCookie.isSecure, nextCookie.expires,
                                nextCookie.status, nextCookie.policy);
  }
  cookiesTreeView.rowCount = cookies.length;

  // sort and display the table
  cookiesTree.treeBoxObject.view = cookiesTreeView;
  /* if (window.arguments[0] == "cookieManagerFromIcon") { // came here by clicking on cookie icon

    // turn off the icon
    var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    observerService.notifyObservers(null, "cookieIcon", "off");

    // unhide the status column and sort by reverse order on that column
    //    Note that the sort routine normally does an ascending sort.  The only
    //    exception is if you sort on the same column more than once in a row.
    //    In that case, the subsequent sorts will be the reverse of the previous ones.
    //    So we are now going to force an initial reverse sort by fooling the sort routine
    //    into thinking that it previously sorted on the status column and in ascending
    //    order.
    document.getElementById("statusCol").removeAttribute("hidden");
    lastCookieSortAscending = true; // force order to have blanks last instead of vice versa
    lastCookieSortColumn = 'status';
    CookieColumnSort('status');

  } else {
    // sort by host column
    CookieColumnSort('rawHost');
  } */
  CookieColumnSort('rawHost');

  // disable "remove all cookies" button if there are no cookies
  /* if (cookies.length == 0) {
    document.getElementById("removeAllCookies").setAttribute("disabled","true");
  } else {
    document.getElementById("removeAllCookies").removeAttribute("disabled");
  } */

  // show policy field if at least one cookie has a policy
  if (showPolicyField) {
    document.getElementById("policyField").removeAttribute("hidden");
  }
}

function GetExpiresString(expires)
{
  if (expires) {
    var date = new Date(1000*expires);
    return gDateService.FormatDateTime("", gDateService.dateFormatLong,
                                       gDateService.timeFormatSeconds, date.getFullYear(),
                                       date.getMonth()+1, date.getDate(), date.getHours(),
                                       date.getMinutes(), date.getSeconds());
  }
  return cookieBundle.getString("AtEndOfSession");
}

function GetStatusString(status)
{
  dump("\nstatus: " + status);
  switch (status) {
    case nsICookie.STATUS_ACCEPTED:
      return cookieBundle.getString("accepted");
    case nsICookie.STATUS_FLAGGED:
      return cookieBundle.getString("flagged");
    case nsICookie.STATUS_DOWNGRADED:
      return cookieBundle.getString("downgraded");
  }
  return "";
}

function GetPolicyString(policy)
{
  switch (policy) {
    case nsICookie.POLICY_NONE:
      return cookieBundle.getString("policyUnstated");
    case nsICookie.POLICY_NO_CONSENT:
      return cookieBundle.getString("policyNoConsent");
    case nsICookie.POLICY_IMPLICIT_CONSENT:
      return cookieBundle.getString("policyImplicitConsent");
    case nsICookie.POLICY_EXPLICIT_CONSENT:
      return cookieBundle.getString("policyExplicitConsent");
    case nsICookie.POLICY_NO_II:
      return cookieBundle.getString("policyNoIICollected");
  }
  return "";
}

function CookieSelected()
{
  var selections = GetTreeSelections(cookiesTree);

  if (selections.length) {
    // document.getElementById("removeCookie").removeAttribute("disabled");
  } else {
    ClearCookieProperties();
    return true;
  }
  var idx = selections[0];

  if (idx >= cookies.length) {
    // Something got out of synch.  See bug 119812 for details
    dump("Tree and viewer state are out of sync! " +
         "Help us figure out the problem in bug 119812");
    return false;
  }

  var props = [
    {id: "ifl_name", value: cookies[idx].name},
    {id: "ifl_value", value: cookies[idx].value}, 
    {id: "ifl_isDomain",
     value: cookies[idx].isDomain ?
            cookieBundle.getString("domainColon") : cookieBundle.getString("hostColon")},
    {id: "ifl_host", value: cookies[idx].host},
    {id: "ifl_path", value: cookies[idx].path},
    {id: "ifl_isSecure",
     value: cookies[idx].isSecure ?
            cookieBundle.getString("yes") : cookieBundle.getString("no")},
    {id: "ifl_expires", value: GetExpiresString(cookies[idx].expires)},
    {id: "ifl_policy", value: GetPolicyString(cookies[idx].policy)}
  ];

  var value, field;

  for (var i = 0; i < props.length; i++)
  {
    field = document.getElementById(props[i].id);
    if ((selections.length > 1) && (props[i].id != "ifl_isDomain")) {
      value = ""; // clear field if multiple selections
    } else {
      value = props[i].value;
    }
    field.value = value;
  }
  return true;
}

function ClearCookieProperties()
{
  var properties = ["ifl_name","ifl_value","ifl_host","ifl_path","ifl_isSecure","ifl_expires","ifl_policy"];

  for (var prop=0; prop<properties.length; prop++) {
    document.getElementById(properties[prop]).value = "";
  }
}

function DeleteCookie()
{
  DeleteSelectedItemFromTree(cookiesTree, cookiesTreeView, cookies, deletedCookies, "removeCookie", "removeAllCookies");
  if (!cookies.length) {
    ClearCookieProperties();
  }
  FinalizeCookieDeletions();
}

function DeleteAllCookies()
{
  ClearCookieProperties();
  DeleteAllFromTree(cookiesTree, cookiesTreeView, cookies, deletedCookies, "removeCookie", "removeAllCookies");
  FinalizeCookieDeletions();
}

function FinalizeCookieDeletions() 
{
  for (var c=0; c<deletedCookies.length; c++) {
    cookiemanager.remove(deletedCookies[c].host,
                         deletedCookies[c].name,
                         deletedCookies[c].path,
                         document.getElementById("checkbox").checked);
  }
  deletedCookies.length = 0;
}

function HandleCookieKeyPress(e)
{
  if (e.keyCode == 46) {
    DeleteCookie();
  }
}

function CookieColumnSort(column)
{
  lastCookieSortAscending = SortTree(cookiesTree, cookiesTreeView, cookies, column, lastCookieSortColumn, lastCookieSortAscending);
  lastCookieSortColumn = column;
}

