/***
  * mzContentBlocker XPCOM component
  * Version: v0.50 (beta quality assured)
  *
  * Copyright (c) 2002-2004 by HJ van Rantwijk - All Rights Reserved!
  * Warning: this is NOT just free/Public Domain/Open Source Software!
  *
  */

/***
  * Constants
  */

const BEHAVIOR_ALLOW             = 1;
const BEHAVIOR_ALLOW_SAME_DOMAIN = 2;
const BEHAVIOR_BLOCK             = 4;
const BEHAVIOR_ASK_ME            = 8;
const BEHAVIOR_USE_REGEX         = 16;

const MIMETYPE_USE_REGEX         = 1;
const MIMETYPE_STORE             = 2;

const UNLISTED_PERMISSION_ENTRY  = -1;

/*
 77   const PRUint32 UNKNOWN_ACTION = 0;
 78   const PRUint32 ALLOW_ACTION = 1;
 79   const PRUint32 DENY_ACTION = 2;
 */

/***
  * URL: http://lxr.mozilla.org/seamonkey/source/content/base/public/nsIContentPolicy.idl
  *
  * const SCRIPT = Components.interfaces.nsIContentPolicy.SCRIPT // 1
  * const IMAGE = Components.interfaces.nsIContentPolicy.IMAGE // 2
  * const STYLESHEET = Components.interfaces.nsIContentPolicy.STYLESHEET // 3
  * const OBJECT = Components.interfaces.nsIContentPolicy.OBJECT // 4
  * const SUBDOCUMENT = Components.interfaces.nsIContentPolicy.SUBDOCUMENT // 5
  * const CONTROL_TAG = Components.interfaces.nsIContentPolicy.CONTROL_TAG // 6
  * const RAW_URL = Components.interfaces.nsIContentPolicy.RAW_URL // 7
  * const DOCUMENT = Components.interfaces.nsIContentPolicy.DOCUMENT // 8
  */

// List with the supported permission types
// var permissionTypes = [ "unsupported", "script", "image", "stylesheet", "object", "frame", "meta", "unsupported", "unsupported" ];
var permissionTypes    = [ "unsupported", "script", "image", "stylesheet", "object", "frame", "meta", "unsupported", "unsupported" ];
var contentId          = null; // this will be initialized with one of the above strings
var behaviorPrefValues = [ -1, -1, -1, -1, -1, -1, -1, -1, -1 ];
var mimePrefValues     = [ -1, -1, -1, -1, -1, -1, -1, -1, -1 ];
var patterns           = new Array();
var kIsEnabled         = false;
var kDebug             = true;
/***
  * Used interfaces
  */

const permissionmanager = Components.classes["@mozilla.org/permissionmanager;1"]
                                    .getService(Components.interfaces.nsIPermissionManager);
const gIOService        = Components.classes["@mozilla.org/network/io-service;1"]
                                    .getService(Components.interfaces.nsIIOService);
const prefs             = Components.classes["@mozilla.org/preferences-service;1"]
                                    .getService(Components.interfaces.nsIPrefBranch);
const promptService     = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                    .getService(Components.interfaces.nsIPromptService);
const cacheService      = Components.classes["@mozilla.org/network/cache-service;1"]
                                    .getService(Components.interfaces.nsICacheService);
const gMIMEService      = Components.classes["@mozilla.org/mime;1"]
                                    .getService(Components.interfaces.nsIMIMEService);
/*** 
  * Preferences
  * pref("multizilla.content-blocker.enabled", "bool", [true/false]);
  * pref("multizilla.content-blocker.behavior.xxx", "int", [1=disabled/2=allow/4=same domain/8=block/16=ask me/32=use regex]/64=keep);
  * pref("multizilla.content-blocker.mimetype.behavior.xxx", "int", [1=disabled/2=enabled/4=ask me/8=use regex/16=keep]);
  * pref("multizilla.content-blocker.mimetypes.for.xxx", "string", "video/quicktime, application/x-shockwave-flash, application/futuresplash")
  */

/***
  * Prefs observer
  */
const cbPrefListener =
{
  domain: "multizilla.content-blocker",
  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;
    if (aPrefName == this.domain + ".enabled") {
      kIsEnabled = prefs.getBoolPref(aPrefName);
      debug("\nPref Changed to: " + kIsEnabled);
    }
    else if (aPrefName.match(this.domain + ".behavior.")) {
      initBehaviorFor(aPrefName, "behavior");
    }
    else if (aPrefName.match(this.domain + ".mimetype.behavior.")) {
      initBehaviorFor(aPrefName, "mimetype.behavior");
    }
    else if (aPrefName.match(this.domain + ".pattern.for.")) {
      initPatternFor(aPrefName);
    }
  }
};
////////////////////////////////////////////////////////////////////////
// The mzContentBlocker class constructor.
////////////////////////////////////////////////////////////////////////
function mzContentBlocker()
{
  for (i in permissionTypes) {
    var regExContainer = (permissionTypes[i] == "unsupported") ? null : [];
    patterns.push(regExContainer);
  }
  addPrefListener(cbPrefListener);
}
////////////////////////////////////////////////////////////////////////
// mzContentBlocker : nsIContentPolicy, nsISupports
////////////////////////////////////////////////////////////////////////
mzContentBlocker.prototype =
{
    getVersionInfo: function () 
    {
      return("0.50a");
	},

    shouldLoad: function (contentType, location, ctxt, win)
    {
      // Sanity checks
      if (!kIsEnabled || (contentType < 1 || contentType > 6) || (location.scheme != 'http' && location.scheme != 'https')) {
        return true;
      }
      contentId = permissionTypes[contentType]; // Match string value for current permission type

      if (contentId != "unsupported") {
        var behavior = parseInt(behaviorPrefValues[contentType]);

        if (behavior > 0) {
          return this.checkPolicyFor(behavior, location, ctxt, win, contentType, contentId);
        }
      }
      return true; // Allow load
    },

    checkPolicyFor: function(aBehavior, aURI, ctxt, aWindow, aContentType, aContentId)
    {
      // var URI = gIOService.newURI(aURI.spec, null, null);
      var capability = this.shouldAllowContent(aBehavior, aURI, aContentType, aContentId, aWindow);

      if (capability == UNLISTED_PERMISSION_ENTRY) {
        if (aBehavior & BEHAVIOR_ASK_ME) {
          capability = this.getUsersPermission(aBehavior, aURI, ctxt, aWindow, aContentType, aContentId, capability);
        }
        else {
          capability = (aBehavior & BEHAVIOR_ALLOW || (aBehavior & BEHAVIOR_ALLOW_SAME_DOMAIN)) ? true : false;
        }
      }
      if (capability == false)
        dump("\n! " + aContentId + " !\nuri: " + aURI.spec);

      return capability; // true or false
    },

    shouldAllowContent: function(aBehavior, aURI, contentType, aContentId, aWindow)
    {
       var permission = this.readPermission(aBehavior, aURI, aContentId, aWindow);
       // dump("\npermission: " + permission);
       if (permission == UNLISTED_PERMISSION_ENTRY && patterns[contentType] != null) {
         for (i in patterns[contentType]) {
           // if (aURI.hostPort.match(patterns[contentType][i])) {
           var regexp = new RegExp();

           if (aURI.spec.match(regexp.compile(patterns[contentType][i]),'i')) {
             // dump("\nPattern match for: " + permissionTypes[contentType] + " : " + patterns[contentType][i] + "\nLoad denied:\n " + aURI.spec);

             if ((aBehavior & BEHAVIOR_ALLOW) || (aBehavior & BEHAVIOR_ALLOW_SAME_DOMAIN)) {
               // dump("\nBEHAVIOR_ALLOW || BEHAVIOR_ALLOW_SAME_DOMAIN");
               return false;
             }
             else if (aBehavior & BEHAVIOR_BLOCK) {
               // dump("\nBEHAVIOR_BLOCK");
               return true;
             }
           }
         }
       }
       // dump("\npermission : " + permission);
       return permission; // true, false or UNLISTED_PERMISSION_ENTRY
    },

    readPermission: function(aBehavior, aURI, aContentId, aWindow)
    {
      // dump("\naBehavior: " + aBehavior + "\naURI: " + aURI.spec + "\naType: " + aContentId);
      var shouldLoad = true;

      if (aURI.scheme == "file") {
        aURI = gIOService.newURI("file:", null, null);
      }
      var permission = permissionmanager.testPermission(aURI, aContentId);

      if (aBehavior & BEHAVIOR_ALLOW_SAME_DOMAIN) {
        shouldLoad = (permission == permissionmanager.DENY_ACTION) ? false : true;
        var browser = this.getBrowserForDocument(aWindow.document);
        // dump("\naURI.hostPort: " + aURI.hostPort);
		// d ump("\nbrowser: " + browser.contentDocument.domain);
        if (aURI.hostPort != browser.contentDocument.domain) {
          // dump("\nContent blocked for other domain " + aURI.hostPort);
          return false;
        }
      }
      if (permission == permissionmanager.UNKNOWN_ACTION) {
        return (UNLISTED_PERMISSION_ENTRY);
      }
      else if (aBehavior & BEHAVIOR_ALLOW) {
        shouldLoad = (permission == permissionmanager.DENY_ACTION) ? false : true;
      }
      else if (aBehavior & BEHAVIOR_BLOCK) {
        shouldLoad = (permission == permissionmanager.ALLOW_ACTION);
      }
      /* if (!shouldLoad)
        dump("\nshouldAllowContent: " + aContentId + " rejected!");
        dump("\nshouldLoad: " + shouldLoad + "\npermission: " + permission); */
      return shouldLoad; // true or false
    },

    getUsersPermission: function (aBehavior, aURI, ctxt, aWindow, aContentType, aContentId, capability)
    {
      if (!ctxt) // Sanity check
        ctxt = aWindow.document;

      var mimeType = this.getMimeTypeFor(aURI, ctxt, aWindow, aContentId);
      mimeType = mimeType || "Undetermined mimeType";
      var domain = ('domain' in aWindow.document) ? aWindow.top.document.domain : "Undetermined";
      var host;

      if (aURI.scheme == "file") {
        host = "file:";
      }
      else {
        host = aURI.hostPort;
      }
      var confirmationMsg = "[ " + domain + " ] requests for: [ " + mimeType + " ] \nfrom [ " + host + " ]\n\nDo you want to Allow or Block this request?";
      var usersResponse = promptService.confirm(aWindow, "Content Blocker for " + aContentId, confirmationMsg);
      // Do we need to update the permissions?
      if (usersResponse != capability && capability != BEHAVIOR_USE_REGEX) {
        this.updatePermissions(aURI, usersResponse, aContentType);
      }
      // Do we need to update the mime types?
      if (mimePrefValues[aContentType] & MIMETYPE_STORE) {
        this.updateMimeTypes(mimeType);
      }
      return usersResponse;
    },

    getMimeTypeFor: function (aURI, ctxt, aWindow, aContentId)
    {
      if (!ctxt) // Sanity check
        return null;

      var mimeType = ('type' in ctxt && ctxt.type) || 
                     ('codeType' in ctxt && ctxt.codeType) || 
                     ('contentType' in ctxt && ctxt.contentType);

      if (!mimeType) {
        try {
          mimeType = gMIMEService.getTypeFromURI(aURI);
        } 
        catch(ex) { // Invalid or unsupported (like .js) file extension used!
          // This works but sucks. I should use 
          if ('src' in ctxt) {
            if (aContentId == 'image') { // Fallback for HTMLImageElement's
              mimeType = "image/unknown";
            }
            else if (aContentId == 'script') { // Fallback for HTMLScriptElement's
              var language = ctxt.getAttribute('language'); // Check language attribute like "javascript(X.X)"

              if (language && language.match(/javascript/i)) {
                mimeType = "text/javascript";
              }
              else { // Check extension for <script src=/.../somefile.js"></script>
                var uri = aURI.QueryInterface(Components.interfaces.nsIURL);
                mimeType = (uri.fileExtension == 'js') ? "unspecified/javascript" : "unspecified/script";
              }
            }
            if (!mimeType) { // Did we find a mimetype?
              mimeType = this.getContentTypeFromHeaders(ctxt.src); // No, check cache entries
            }
          }
          else if ('location' in ctxt) { // documents
            mimeType = this.getContentTypeFromHeaders(ctxt.location);
          }
        }
      }
      mimeType = (mimeType || aContentId + "/undetermined");
      // dump("\nmimeType: " + mimeType);
      return mimeType;
    },

    getContentTypeFromHeaders: function (aURL)
    {
      var match = null;

      try {
        var httpCacheSession = cacheService.createSession("HTTP", 0, true);
        var cacheEntryDescriptor = httpCacheSession.openCacheEntry(aURL, Components.interfaces.nsICache.ACCESS_READ, false); // open for READ, in non-blocking mode
        var headers = cacheEntryDescriptor.getMetaDataElement("response-head");
        match = /^Content-Type:\s*(.*?)\s*(?:\;|$)/mi.exec(headers);
        // dump("\nmatch[1]: "+match[1]);
      }
      catch (ex) {
       dump("\nRequested Entry Not Cached: " + aURL);
      }
      return (match ? match[1] : match);
    },

    getLastActiveBrowserWindow: function() 
    {
      var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
      var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
      var window = windowManagerInterface.getMostRecentWindow("navigator:browser");
      return window;
    },

    getBrowserForDocument: function(aDocument)
    {
      // We currently check the first browser window only!
      var window = this.getLastActiveBrowserWindow();
      var browsers = window.gBrowser.browsers;

      for (var i = 0; i < browsers.length; i++) {
        if (browsers[i].contentDocument == aDocument) {
          // dump("\nBrowser Found!");
          return browsers[i];
        }
        if ("frames" in browsers[i].contentWindow) {
          var frames = browsers[i].contentWindow.frames.length;
          // dump("\nHas "+frames+" Frames!");
          for (frame = 0; frame < frames; frame++) {
            // dump("\nComparing Frame: "+frame);
            if (browsers[i].contentWindow.frames[frame].document == aDocument) {
              // dump("\nBrowser Found via Frames!");
              return browsers[i];
            }
          }
        }
      }
      // dump("\nBrowser Lookup Failed!");
      return null;
    },

    updatePermissions: function (aURI, aShouldLoad, aContentId)
    {
      var permission = aShouldLoad ? permissionmanager.ALLOW_ACTION : permissionmanager.DENY_ACTION;
      // dump("\npermissionmanager.add("+permission+")");

      if (aURI.scheme == "file") {
        aURI = gIOService.newURI("file:", null, null);
      }
      permissionmanager.add(aURI, aContentId, permission);
    },

    updateMimeTypes: function (aMimeType)
    {
      // debug("\nin this.updateMimeTypes()");
      if (!aMimeType) // Sanity check
        return;

      /* if (prefs.getBoolPref("multizilla.content-blocker.mimetypes.keep"))
        dump("\nStore MimeType"); */
    },

    setDebugStateTo: function (aState) 
    {
      kDebug = aState;
	},

    shouldProcess: function(contentType, location, ctxt, win)
    {
      return true;
    },
    // This "class" supports nsIContentPolicy and nsISupports.
    QueryInterface: function (iid)
    {
      if (!iid.equals(Components.interfaces.nsIContentPolicy) &&
          !iid.equals(Components.interfaces.nsISupports)) {
          throw Components.results.NS_ERROR_NO_INTERFACE;
      }
      return this;
    }
}
////////////////////////////////////////////////////////////////////////
//  mzContentBlocker Module (for XPCOM registration)
////////////////////////////////////////////////////////////////////////
var mzContentBlockerModule =
{
  firstTime:     true,
  mClassName:    "MultiZilla Content Blocker",
  mContractID:   "@multizilla.mozdev.org/contentBlocker;1",
  mClassID:      Components.ID("bc1f905a-07ac-434d-97cf-6b2a996bf00d"),
  // getClassObject: Return this component's factory object.
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aCID.equals(this.mClassID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return this.mFactory;
  },

  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    if (kDebug)
      debug("Registering...");
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(this.mClassID, this.mClassName, this.mContractID, aFileSpec, aLocation, aType);
    this.getCategoryManager().addCategoryEntry("content-policy", this.mContractID, "", true, true);
  },

  unregisterSelf: function(aCompMgr, aFileSpec, aLocation)
  {
    if (kDebug)
      debug("UnRegistering...");
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(this.mClassID, aFileSpec);
    this.getCategoryManager().deleteCategoryEntry("content-policy", this.mContractID, true);
  },

  canUnload: function(aCompMgr)
  {
    return true;
  },

  getCategoryManager: function()
  {
    return Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
  },
  //////////////////////////////////////////////////////////////////////
  //  mFactory : nsIFactory
  //////////////////////////////////////////////////////////////////////
  mFactory: {
    createInstance: function (aOuter, aIID) {
      if (aOuter != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      return (new mzContentBlocker()).QueryInterface(aIID);
    }
  },
};

function NSGetModule(aCompMgr, aFileSpec) 
{
  return mzContentBlockerModule;
}

function addPrefListener(observer)
{
  try {
    var pbi = prefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
        pbi.addObserver(observer.domain, observer, false);
  } catch(ex) {
    debug("Pref Observer Failed!");
  }
}

function initBehaviorFor(aPrefName, aPrefSelector)
{
  // multizilla.content-blocker.behavior.type -> type
  var contentType = aPrefName.replace(/[-\w]*\./ig, '');
  var prefArray = (aPrefSelector == "behavior") ? behaviorPrefValues : mimePrefValues;

  for (i in permissionTypes) {
    if (permissionTypes[i] == contentType) {
      // dump("\naPrefSelector: " + aPrefSelector);
      prefArray[i] = prefs.getIntPref(aPrefName);
    }
  }
}

function initPatternFor(aPrefName)
{
  // multizilla.content-blocker.behavior.type -> type
  var contentType = aPrefName.replace(/[-\w]*\./ig, '');

  for (i in permissionTypes) {
    if (permissionTypes[i] == contentType) {
      var pattern = prefs.getComplexValue(aPrefName, Components.interfaces.nsISupportsString).data;
      pattern = pattern.replace(/\s/g, '').replace(/\./g, '\.+').replace(/\*/g, '.*');
      dump("\nPattern Found for " + contentType + " : " + pattern);
      patterns[i] = pattern.split(',');
      break;
    }
  }
}

////////////////////////////////////////////////////////////////////////
//  Debug Helper function
////////////////////////////////////////////////////////////////////////
if (!kDebug)
  debug = function(txt) {};
else
  debug = function(txt) {dump("\nContent Blocker: " + txt + "\n");};

